'use strict';
var through = require('through2');
var pp = require('preprocess');
var html2json = require('html2json').html2json;
var json2html = require('html2json').json2html;
var cheerio = require('cheerio');
var minify = require('html-minifier').minify;
const Entities = require('html-entities').XmlEntities;
const entities = new Entities();

var obj = {
    replace:{
        view:['body','article','aside','ul','li','ol','caption','dd','dl','dt','footer','header','nav','section','table','thead','tbody','tr','td','th','h1','h2','h3','h4','h5','h6','div','p','span'],
        image:['img'],
        navigator:['a'],
        text:['s','b','i','strong','font']
    },
    css2wxss: function (options) {
        var _this = this;
        return through.obj(function (file, enc, cb) {
            if (file.isNull()) {
                this.push(file);
                return cb();
            }

            if (file.isStream()) {
                this.emit('error', new gutil.PluginError(PLUGIN_NAME, 'Streaming not supported'));
                return cb();
            }

            var css = pp.preprocess(file.contents.toString('utf8'), options || {});
            css = css.replace(/\simg/g,' image');
            css = css.replace(/\sinput/g,' input');
            file.contents = new Buffer(css);
            this.push(file);

            cb();
        });
    },
    html2wxml: function (options) {
        if(options.collapse == undefined) options.collapse = true;
        var _this = this;
        return through.obj(function (file, enc, cb) {
            if (file.isNull()) {
                this.push(file);
                return cb();
            }

            if (file.isStream()) {
                this.emit('error', new gutil.PluginError(PLUGIN_NAME, 'Streaming not supported'));
                return cb();
            }

            var html = pp.preprocess(file.contents.toString('utf8'), options || {});
            var $ = cheerio.load(html);
            var body = $('body').html();
            body = minify(body, {
                collapseWhitespace: options.collapse
            });
            var json = html2json(body);
            html = json2html(_this.process(json));
            html = entities.decode(html);
            file.contents = new Buffer(html);
            this.push(file);

            cb();
        });
    },
    process: function (json) {
        var _this = this;
        var result = {
            child: []
        };
        for (var i in json) {
            switch (i) {
                case 'node':
                    result.node = json.node;
                    break;
                case 'tag':
                    //替换TAG
                    result.tag = _this.replaceTag(json.tag);
                    break;
                case 'attr':
                    result.attr = json.attr;
                    break;
                case 'child':
                    for (var j in json.child) {
                        result.child.push(_this.process(json.child[j]))
                    }
                    break;
                default:
                    result[i] = json[i];
                    break;
            }
        }
        return result;
    },
    replaceTag:function(tag){
        if(tag.in_array(this.replace.view)){
            return 'view';
        }
        if(tag.in_array(this.replace.image)){
            return 'image';
        }
        if(tag.in_array(this.replace.navigator)){
            return 'navigator';
        }
        if(tag.in_array(this.replace.text)){
            return 'text';
        }
        return tag;
    }
}

var in_array = function(arr){
    if(typeof(arr)!='object'){
        throw "arguments is not Array";
    }
    for(var i=0,k=arr.length;i<k;i++){
        if(this==arr[i]){
            return true;
        }
    }
    return false;
}
String.prototype.in_array = in_array;
Number.prototype.in_array = in_array;

module.exports = obj;